=== modified file 'drizzled/item/subselect.cc'
--- drizzled/item/subselect.cc	2009-07-29 18:35:48 +0000
+++ drizzled/item/subselect.cc	2009-07-31 20:29:49 +0000
@@ -134,10 +134,11 @@
 
 void Item_in_subselect::cleanup()
 {
-  while (left_expr_cache.empty() == false)
+  if (left_expr_cache)
   {
-    delete left_expr_cache.back();
-    left_expr_cache.pop_back();
+    left_expr_cache->delete_elements();
+    delete left_expr_cache;
+    left_expr_cache= NULL;
   }
   first_execution= true;
   Item_subselect::cleanup();
@@ -306,11 +307,11 @@
     - on a cost-based basis, that takes into account the cost of a cache
       lookup, the cache hit rate, and the savings per cache hit.
   */
-  if (left_expr_cache.empty() == true && exec_method == MATERIALIZATION)
+  if (!left_expr_cache && exec_method == MATERIALIZATION)
     init_left_expr_cache();
 
   /* If the new left operand is already in the cache, reuse the old result. */
-  if (left_expr_cache.empty() == false && test_if_item_cache_changed(left_expr_cache) < 0)
+  if (left_expr_cache && test_if_item_cache_changed(*left_expr_cache) < 0)
   {
     /* Always compute IN for the first row as the cache is not valid for it. */
     if (!first_execution)
@@ -704,7 +705,7 @@
 
 Item_in_subselect::Item_in_subselect(Item * left_exp,
 				     Select_Lex *select_lex):
-  Item_exists_subselect(), first_execution(true),
+  Item_exists_subselect(), left_expr_cache(0), first_execution(true),
   optimizer(0), pushed_cond_guards(NULL), exec_method(NOT_TRANSFORMED),
   upper_item(0)
 {
@@ -1833,16 +1834,16 @@
   if (end_select == end_send_group || end_select == end_write_group)
     use_result_field= true;
 
-  left_expr_cache.clear();
+  if (!(left_expr_cache= new List<Cached_item>))
+    return true;
 
   for (uint32_t i= 0; i < left_expr->cols(); i++)
   {
     Cached_item *cur_item_cache= new_Cached_item(session,
                                                  left_expr->element_index(i),
                                                  use_result_field);
-    if (cur_item_cache == NULL)
+    if (!cur_item_cache || left_expr_cache->push_front(cur_item_cache))
       return true;
-    left_expr_cache.push_back(cur_item_cache);
   }
   return false;
 }

=== modified file 'drizzled/item/subselect.h'
--- drizzled/item/subselect.h	2009-07-16 21:19:14 +0000
+++ drizzled/item/subselect.h	2009-07-31 20:29:49 +0000
@@ -26,7 +26,6 @@
 #include <drizzled/comp_creator.h>
 #include <drizzled/item/ref.h>
 #include <drizzled/item/field.h>
-#include <vector>
 
 class Select_Lex;
 class Select_Lex_Unit;
@@ -283,7 +282,7 @@
     Cache of the left operand of the subquery predicate. Allocated in the
     runtime memory root, for each execution, thus need not be freed.
   */
-  std::vector<Cached_item*> left_expr_cache;
+  List<Cached_item> *left_expr_cache;
   bool first_execution;
 
   /*
@@ -333,7 +332,7 @@
 
   Item_in_subselect(Item * left_expr, Select_Lex *select_lex);
   Item_in_subselect()
-    :Item_exists_subselect(), first_execution(true),
+    :Item_exists_subselect(), left_expr_cache(0), first_execution(true),
     optimizer(0), abort_on_null(0), pushed_cond_guards(NULL),
     exec_method(NOT_TRANSFORMED), upper_item(0)
   {}

=== modified file 'drizzled/item/sum.cc'
--- drizzled/item/sum.cc	2009-07-28 00:46:31 +0000
+++ drizzled/item/sum.cc	2009-07-31 20:29:49 +0000
@@ -216,8 +216,9 @@
     Check that non-aggregated fields and sum functions aren't mixed in the
     same select in the ONLY_FULL_GROUP_BY mode.
   */
-  if (outer_fields.size())
+  if (outer_fields.elements)
   {
+    Item_field *field;
     /*
       Here we compare the nesting level of the select to which an outer field
       belongs to with the aggregation level of the sum function. All fields in
@@ -244,10 +245,10 @@
         select the field belongs to. If there are some then an error is
         raised.
     */
-    vector<Item_field*>::iterator field= outer_fields.begin();
-    while (field != outer_fields.end())
+    List_iterator<Item_field> of(outer_fields);
+    while ((field= of++))
     {
-      Select_Lex *sel= (*field)->cached_table->select_lex;
+      Select_Lex *sel= field->cached_table->select_lex;
       if (sel->nest_level < aggr_level)
       {
         if (in_sum_func)
@@ -256,7 +257,7 @@
             Let upper function decide whether this field is a non
             aggregated one.
           */
-          in_sum_func->outer_fields.push_back(*field);
+          in_sum_func->outer_fields.push_back(field);
         }
         else
         {
@@ -272,7 +273,6 @@
         return true;
       }
     }
-    ++field;
   }
   aggr_sel->full_group_by_flag.set(SUM_FUNC_USED);
   update_used_tables();

=== modified file 'drizzled/item/sum.h'
--- drizzled/item/sum.h	2009-07-28 00:46:31 +0000
+++ drizzled/item/sum.h	2009-07-31 20:29:49 +0000
@@ -29,7 +29,6 @@
 #include <drizzled/item/field.h>
 #include <drizzled/item/bin_string.h>
 #include <mystrings/m_string.h>
-#include <vector>
 
 extern "C"
 int group_concat_key_cmp_with_distinct(void* arg, const void* key1,
@@ -263,7 +262,7 @@
     directly or indirectly used under this function it as it's unclear
     at the moment of fixing outer field whether it's aggregated or not.
   */
-  std::vector<Item_field*> outer_fields;
+  List<Item_field> outer_fields;
 
 protected:
   table_map used_tables_cache;

=== modified file 'drizzled/join.cc'
--- drizzled/join.cc	2009-07-29 18:35:48 +0000
+++ drizzled/join.cc	2009-07-31 20:29:49 +0000
@@ -1512,6 +1512,7 @@
       setup_copy_fields(session, &curr_join->tmp_table_param,
       items3, tmp_fields_list3, tmp_all_fields3,
       curr_fields_list->elements, *curr_all_fields);
+      tmp_table_param.save_copy_funcs= curr_join->tmp_table_param.copy_funcs;
       tmp_table_param.save_copy_field= curr_join->tmp_table_param.copy_field;
       tmp_table_param.save_copy_field_end= curr_join->tmp_table_param.copy_field_end;
       curr_join->tmp_all_fields3= tmp_all_fields3;
@@ -1519,6 +1520,7 @@
     }
     else
     {
+      curr_join->tmp_table_param.copy_funcs= tmp_table_param.save_copy_funcs;
       curr_join->tmp_table_param.copy_field= tmp_table_param.save_copy_field;
       curr_join->tmp_table_param.copy_field_end= tmp_table_param.save_copy_field_end;
     }
@@ -1883,18 +1885,12 @@
   {
     if (tmp_join)
       tmp_table_param.copy_field= 0;
-
-    while (group_fields.empty() == false)
-    {
-      delete group_fields.back();
-      group_fields.pop_back();
-    }
-
+    group_fields.delete_elements();
     /*
       We can't call delete_elements() on copy_funcs as this will cause
       problems in free_elements() as some of the elements are then deleted.
     */
-    tmp_table_param.copy_funcs.clear();
+    tmp_table_param.copy_funcs.empty();
     /*
       If we have tmp_join and 'this' JOIN is not tmp_join and
       tmp_table_param.copy_field's  of them are equal then we have to remove
@@ -1906,9 +1902,10 @@
         tmp_join->tmp_table_param.copy_field ==
         tmp_table_param.copy_field)
     {
-      tmp_join->tmp_table_param.copy_field= NULL;
-      tmp_join->tmp_table_param.save_copy_field= NULL;
+      tmp_join->tmp_table_param.copy_field=
+        tmp_join->tmp_table_param.save_copy_field= 0;
     }
+    tmp_table_param.cleanup();
   }
   return;
 }
@@ -2934,7 +2931,7 @@
 */
 static bool make_group_fields(JOIN *main_join, JOIN *curr_join)
 {
-  if (main_join->group_fields_cache.empty() == false)
+  if (main_join->group_fields_cache.elements)
   {
     curr_join->group_fields= main_join->group_fields_cache;
     curr_join->sort_and_group= 1;
@@ -3026,7 +3023,9 @@
 }
 
 /**
-  Get a list of buffers for saving last group.
+  Get a list of buffers for saveing last group.
+
+  Groups are saved in reverse order for easyer check loop.
 */
 static bool alloc_group_fields(JOIN *join,order_st *group)
 {
@@ -3035,9 +3034,8 @@
     for (; group ; group=group->next)
     {
       Cached_item *tmp=new_Cached_item(join->session, *group->item, false);
-      if (!tmp)
+      if (!tmp || join->group_fields.push_front(tmp))
         return true;
-      join->group_fields.push_back(tmp);
     }
   }
   join->sort_and_group=1;     /* Mark for do_select */

=== modified file 'drizzled/join.h'
--- drizzled/join.h	2009-07-29 18:35:48 +0000
+++ drizzled/join.h	2009-07-31 20:29:49 +0000
@@ -140,8 +140,8 @@
   nested_join_map cur_embedding_map;
 
   double best_read;
-  std::vector<Cached_item*> group_fields;
-  std::vector<Cached_item*> group_fields_cache;
+  List<Cached_item> group_fields;
+  List<Cached_item> group_fields_cache;
   Table *tmp_table;
   /** used to store 2 possible tmp table of SELECT */
   Table *exec_tmp_table1;
@@ -285,6 +285,7 @@
     if (&fields_list != &fields_arg) /* only copy if not same*/
       fields_list= fields_arg;
     memset(&keyuse, 0, sizeof(keyuse));
+    tmp_table_param.init();
     tmp_table_param.end_write_records= HA_POS_ERROR;
     rollup.state= ROLLUP::STATE_NONE;
   }
@@ -373,6 +374,7 @@
     if (&fields_list != &fields_arg) /* only copy if not same*/
       fields_list= fields_arg;
     memset(&keyuse, 0, sizeof(keyuse));
+    tmp_table_param.init();
     tmp_table_param.end_write_records= HA_POS_ERROR;
     rollup.state= ROLLUP::STATE_NONE;
   }

=== modified file 'drizzled/session.cc'
--- drizzled/session.cc	2009-07-28 22:00:14 +0000
+++ drizzled/session.cc	2009-07-31 20:29:49 +0000
@@ -1586,6 +1586,30 @@
   return false;
 }
 
+/****************************************************************************
+  Tmp_Table_Param
+****************************************************************************/
+
+void Tmp_Table_Param::init()
+{
+  field_count= sum_func_count= func_count= hidden_field_count= 0;
+  group_parts= group_length= group_null_parts= 0;
+  quick_group= 1;
+  table_charset= 0;
+  precomputed_group_by= 0;
+  bit_fields_as_long= 0;
+}
+
+void Tmp_Table_Param::cleanup(void)
+{
+  /* Fix for Intel compiler */
+  if (copy_field)
+  {
+    delete [] copy_field;
+    save_copy_field= copy_field= 0;
+  }
+}
+
 void Session::send_kill_message() const
 {
   int err= killed_errno();

=== modified file 'drizzled/show.cc'
--- drizzled/show.cc	2009-07-29 00:01:38 +0000
+++ drizzled/show.cc	2009-07-31 20:29:49 +0000
@@ -2514,7 +2514,9 @@
     field_count++;
     ++iter;
   }
-  Tmp_Table_Param *tmp_table_param= new Tmp_Table_Param;
+  Tmp_Table_Param *tmp_table_param =
+    (Tmp_Table_Param*) (session->alloc(sizeof(Tmp_Table_Param)));
+  tmp_table_param->init();
   tmp_table_param->table_charset= cs;
   tmp_table_param->field_count= field_count;
   tmp_table_param->schema_table= 1;

=== modified file 'drizzled/sql_base.cc'
--- drizzled/sql_base.cc	2009-07-31 20:19:26 +0000
+++ drizzled/sql_base.cc	2009-07-31 20:29:49 +0000
@@ -4611,8 +4611,9 @@
   Item *value, *fld;
   Item_field *field;
   Table *table= 0;
-  vector<Table*> tbl_list;
+  List<Table> tbl_list;
   bool abort_on_warning_saved= session->abort_on_warning;
+  tbl_list.empty();
 
   /*
     Reset the table->auto_increment_field_not_null as it is valid for
@@ -4655,16 +4656,16 @@
   }
   /* Update virtual fields*/
   session->abort_on_warning= false;
-  if (tbl_list.empty() == false)
+  if (tbl_list.head())
   {
+    List_iterator_fast<Table> t(tbl_list);
     Table *prev_table= 0;
-    for (vector<Table*>::iterator it= tbl_list.begin(); it != tbl_list.end(); ++it)
+    while ((table= t++))
     {
       /*
         Do simple optimization to prevent unnecessary re-generating
         values for virtual fields
       */
-      table= *it;
       if (table != prev_table)
         prev_table= table;
     }
@@ -4707,9 +4708,10 @@
   Item *value;
   Table *table= 0;
   Field *field;
-  vector<Table*> tbl_list;
+  List<Table> tbl_list;
   bool abort_on_warning_saved= session->abort_on_warning;
 
+  tbl_list.empty();
   /*
     Reset the table->auto_increment_field_not_null as it is valid for
     only one row.
@@ -4735,16 +4737,16 @@
   }
   /* Update virtual fields*/
   session->abort_on_warning= false;
-  if (tbl_list.empty() == false)
+  if (tbl_list.head())
   {
+    List_iterator_fast<Table> t(tbl_list);
     Table *prev_table= 0;
-    for (vector<Table*>::iterator it= tbl_list.begin(); it != tbl_list.end(); ++it)
+    while ((table= t++))
     {
       /*
         Do simple optimization to prevent unnecessary re-generating
         values for virtual fields
       */
-      table= *it;
       if (table != prev_table)
       {
         prev_table= table;

=== modified file 'drizzled/sql_select.cc'
--- drizzled/sql_select.cc	2009-07-29 18:35:48 +0000
+++ drizzled/sql_select.cc	2009-07-31 20:29:49 +0000
@@ -6743,15 +6743,16 @@
   @return -1 if no item changed
   @return index of the first item that changed
 */
-int test_if_item_cache_changed(vector<Cached_item*> &list)
+int test_if_item_cache_changed(List<Cached_item> &list)
 {
-  int idx= -1;
-  int size= (int)list.size();
+  List_iterator<Cached_item> li(list);
+  int idx= -1,i;
+  Cached_item *buff;
 
-  for (int i= size - 1; i >= 0; --i)
+  for (i=(int) list.elements-1 ; (buff=li++) ; i--)
   {
-    if (list[i]->cmp())
-      idx= i;
+    if (buff->cmp())
+      idx=i;
   }
   return(idx);
 }
@@ -6798,14 +6799,14 @@
   res_selected_fields.empty();
   res_all_fields.empty();
   List_iterator_fast<Item> itr(res_all_fields);
-  vector<Item*> extra_funcs;
+  List<Item> extra_funcs;
   uint32_t i, border= all_fields.elements - elements;
 
   if (param->field_count &&
       !(copy=param->copy_field= new CopyField[param->field_count]))
     goto err2;
 
-  param->copy_funcs.clear();
+  param->copy_funcs.empty();
   for (i= 0; (pos= li++); i++)
   {
     Field *field;
@@ -6829,17 +6830,17 @@
       {
         if (!(pos= new Item_copy_string(pos)))
           goto err;
-
-          /*
-            Item_copy_string::copy for function can call
-            Item_copy_string::val_int for blob via Item_ref.
-            But if Item_copy_string::copy for blob isn't called before,
-            it's value will be wrong
-            so let's insert Item_copy_string for blobs in the beginning of
-            copy_funcs
-            (to see full test case look at having.test, BUG #4358)
-          */
-        param->copy_funcs.push_front(pos);
+            /*
+              Item_copy_string::copy for function can call
+              Item_copy_string::val_int for blob via Item_ref.
+              But if Item_copy_string::copy for blob isn't called before,
+              it's value will be wrong
+              so let's insert Item_copy_string for blobs in the beginning of
+              copy_funcs
+              (to see full test case look at having.test, BUG #4358)
+            */
+        if (param->copy_funcs.push_front(pos))
+          goto err;
       }
       else
       {
@@ -6884,12 +6885,11 @@
         goto err;
       if (i < border)                           // HAVING, order_st and GROUP BY
       {
-        extra_funcs.push_back(pos);
-      }
-      else
-      {
-        param->copy_funcs.push_back(pos);
-      }
+        if (extra_funcs.push_back(pos))
+          goto err;
+      }
+      else if (param->copy_funcs.push_back(pos))
+        goto err;
     }
     res_all_fields.push_back(pos);
     ref_pointer_array[((i < border)? all_fields.elements-i-1 : i-border)]=
@@ -6904,8 +6904,7 @@
     Put elements from HAVING, order_st BY and GROUP BY last to ensure that any
     reference used in these will resolve to a item that is already calculated
   */
-  param->copy_funcs.insert(param->copy_funcs.end(),
-                           extra_funcs.begin(), extra_funcs.end() );
+  param->copy_funcs.concat(&extra_funcs);
 
   return(0);
 
@@ -6931,13 +6930,10 @@
   for (; ptr != end; ptr++)
     (*ptr->do_copy)(ptr);
 
-  deque<Item*>::iterator it= param->copy_funcs.begin();
-  while (it != param->copy_funcs.end())
-  {
-    Item_copy_string *item= (Item_copy_string*)*it;
+  List_iterator_fast<Item> it(param->copy_funcs);
+  Item_copy_string *item;
+  while ((item = (Item_copy_string*) it++))
     item->copy();
-    ++it;
-  }
 }
 
 /**

=== modified file 'drizzled/sql_select.h'
--- drizzled/sql_select.h	2009-07-29 18:35:48 +0000
+++ drizzled/sql_select.h	2009-07-31 20:29:49 +0000
@@ -318,7 +318,7 @@
 bool error_if_full_join(JOIN *join);
 int safe_index_read(JoinTable *tab);
 COND *remove_eq_conds(Session *session, COND *cond, Item::cond_result *cond_value);
-int test_if_item_cache_changed(std::vector<Cached_item*> &list);
+int test_if_item_cache_changed(List<Cached_item> &list);
 
 void print_join(Session *session, String *str,
                 List<TableList> *tables, enum_query_type);

=== modified file 'drizzled/sql_union.cc'
--- drizzled/sql_union.cc	2009-07-29 18:35:48 +0000
+++ drizzled/sql_union.cc	2009-07-31 20:29:49 +0000
@@ -119,6 +119,7 @@
                                   bool bit_fields_as_long)
 {
   assert(table == 0);
+  tmp_table_param.init();
   tmp_table_param.field_count= column_types->elements;
   tmp_table_param.bit_fields_as_long= bit_fields_as_long;
 

=== modified file 'drizzled/sql_update.cc'
--- drizzled/sql_update.cc	2009-07-23 21:28:28 +0000
+++ drizzled/sql_update.cc	2009-07-31 20:29:49 +0000
@@ -1009,7 +1009,8 @@
   table_count=  update_tables.size();
 
   tmp_tables = (Table**) session->calloc(sizeof(Table *) * table_count);
-  tmp_table_param= new Tmp_Table_Param[table_count];
+  tmp_table_param = (Tmp_Table_Param*) session->calloc(sizeof(Tmp_Table_Param) *
+						   table_count);
   fields_for_table= (List_item **) session->alloc(sizeof(List_item *) *
 					      table_count);
   values_for_table= (List_item **) session->alloc(sizeof(List_item *) *
@@ -1228,13 +1229,11 @@
     {
       if (tmp_tables[cnt])
       {
-        tmp_tables[cnt]->free_tmp_table(session);
+	tmp_tables[cnt]->free_tmp_table(session);
+	tmp_table_param[cnt].cleanup();
       }
     }
   }
-
-  delete [] tmp_table_param;
-  
   if (copy_field)
     delete [] copy_field;
   session->count_cuted_fields= CHECK_FIELD_IGNORE;		// Restore this setting

=== modified file 'drizzled/tmp_table_param.h'
--- drizzled/tmp_table_param.h	2009-07-28 00:43:25 +0000
+++ drizzled/tmp_table_param.h	2009-07-31 20:29:49 +0000
@@ -21,15 +21,13 @@
 #ifndef DRIZZLED_TMP_TABLE_PARAM_H
 #define DRIZZLED_TMP_TABLE_PARAM_H
 
-#include <deque>
-
 /*
   Param to create temporary tables when doing SELECT:s
   NOTE
     This structure is copied using memcpy as a part of JOIN.
 */
 
-class Tmp_Table_Param
+class Tmp_Table_Param :public Sql_alloc
 {
 private:
   /* Prevent use of these (not safe because of lists and copy_field) */
@@ -38,7 +36,8 @@
 
 public:
   KEY *keyinfo;
-  std::deque<Item*> copy_funcs;
+  List<Item> copy_funcs;
+  List<Item> save_copy_funcs;
   CopyField *copy_field, *copy_field_end;
   CopyField *save_copy_field, *save_copy_field_end;
   unsigned char	    *group_buff;
@@ -76,37 +75,22 @@
   bool bit_fields_as_long;
 
   Tmp_Table_Param()
-    : keyinfo(NULL),
-      copy_field(NULL),
-      copy_field_end(NULL),
-      save_copy_field(NULL),
-      save_copy_field_end(NULL),
-      group_buff(NULL),
-      items_to_copy(0),
-      recinfo(NULL),
-      start_recinfo(NULL),
-      end_write_records(0),
-      field_count(0),
-      sum_func_count(0),
-      func_count(0),
-      hidden_field_count(0),
-      group_parts(0),
-      group_length(0),
-      group_null_parts(0),
-      quick_group(1),
-      using_indirect_summary_function(false),
-      schema_table(false),
-      precomputed_group_by(false),
-      force_copy_fields(false),
-      convert_blob_length(0),
-      table_charset(NULL),
-      bit_fields_as_long(false)
+    :copy_field(0),
+    group_parts(0),
+    group_length(0),
+    group_null_parts(0),
+    schema_table(false),
+    precomputed_group_by(false),
+    force_copy_fields(false),
+    convert_blob_length(0),
+    bit_fields_as_long(false)
   {}
-
   ~Tmp_Table_Param()
   {
-    delete [] copy_field;
+    cleanup();
   }
+  void init(void);
+  void cleanup(void);
 };
 
 #endif /* DRIZZLED_TMP_TABLE_PARAM_H */

